/****************************************************************************
 *  Copyright (C) 2015 Helge Kortendieck  <alan@kasmithee.de>				*
 *																			*
 *	lame - decode to wav and re-encode to mp3								*
 *	lame will be called via qprocess										*
 *	In general, directory checks can be omitted, as they were done during	*
 *	setting the directories. If things have changed on the mean time		*
 *	(unlikely) lame will bail out. Same goes for file access errors - 		*
 * 	depending on lame as well.												*
 * 																			*
 *	decoding:	input: 	- source directory of mp3s							*
 * 						- list with mp3s to decode, name only				*
 * 						- an output directory for the wav files				*
 *				output:	the wav files in the output directory				*
 *				return:	a list with files that could not be decoded or an	*
 *						error if there was nothing to do, no files,			*
 *						permission problems... . Empty list = full success	*
 *																			*
 *				lame will try to decode all mp3s from the input directory	*
 *				to wavs in the output directory, filenames preserved.		*
 *				If all goes well, the return list is empty, otherwise it	*
 *				will hold a list with files that failed and the error		*
 *				code, if any or any other error, like no mp3s or directory	*
 *				permission problems.										*
 *																			*
 *	encoding:	input:	- directory with (normalized!) wav files to process *
 * 						- list with wavs to encode, name only				*
 *						- output directory for the mp3 files				*
 *				output:	the mp3 files in the output directory				*
 *				return:	a list with files that could not be encoded or an	*
 *						error if there was nothing to do, no files,			*
 *						permisssion problems... . Empty list = full success *
 *																			*
 *				lame will try to encode all wavs from the input directory	*
 *				to mp3s in the output directory, filenames preserved.		*
 *				mp3 quality level will be fetched from the program settings *
 *				on each run. If all goes well, the return list is empty,	*
 *				otherwise it will hold a list with files that failed and	*
 *				the error code if any or any other error, like no wavs or	*
 *				directory permission problems.								*
 *																			*
 *	This program is free software; you can redistribute it and/or modify	*
 *	it under the terms of the GNU General Public License as published by	*
 *	the Free Software Foundation; either version 2 of the License, or		*
 *	(at your option) any later version.										*
 ****************************************************************************/

#include "lame.h"


lame::lame(QObject *parent)
	: QObject(parent)
{
	settingsRead();
}

lame::~lame()
{
}

void lame::settingsRead()
{
	QSettings settings("normalizetogo", "normalizetogo");

	settings.beginGroup("lame");
		sLameQuality = settings.value("lamequality", "--preset extreme").toString();
	settings.endGroup();
}

void lame::lameDo(const QString& sD, const QStringList& slD, const QString& sD2, int iD, bool bD, bool bD2, bool bD3)
{			// src dir, file list with mp3s/wavs to process (name), output dir, command 1=decode/2=encode, wav do not recreate/randomize, del spaces, shorten to 8
	sDirSrc = sD;
	slFilesToProc = slD;
	sDirOut = sD2;
	iLameCmd = iD;
	bWavDoNotReC = bD;							// overloaded with randomize files when re-encoding wav to mp3
	bDelSpaces = bD2;
	bShorten = bD3;
	slReturn.clear();

	if(iLameCmd != 1 and iLameCmd != 2) {
//cout << "lameDo: reached ! " << endl;										// *****  debug  *****
		slReturn.append(tr("Wrong command, not 1 (decode) or 2 (encode)"));
		procLoopEnded();
	}

	iNumFilesToProc = slFilesToProc.count();							// # of files to process
	iLoop = -1;
	bSkip = false;
	liLFilePairs.clear();
	if (iLameCmd == 2) {
		if(bWavDoNotReC == true) {									// overloaded with randomize files when re-encoding wav to mp3
			qsrand(QTime::currentTime().msec());					// initialize random generator
		}
	}
	procLoop();
}

void lame::procLoop()						// run lame as often as it takes to decode all mp3s to wavs or encode wavs to mp3s
{
	iLoop += 1;
	if (iLoop < iNumFilesToProc) {
		slArgs.clear();

		if (iLameCmd == 1) {									// add argument: lame command
			slArgs.append("--decode");							// decode to wav
		} else if(iLameCmd == 2) {
			slArgs.append("--alt-preset");						// encode to mp3
			slArgs.append("extreme");
		} else {
			slReturn.append(tr("Unknown lame command: %1 . Aborting").arg(QString::number(iLameCmd)));
		}

		sFileSrc = sDirSrc + slFilesToProc.at(iLoop);
		slArgs.append(sFileSrc);							// add argument: source file, full path n name
		QFileInfo fiL(sFileSrc);

		if (iLameCmd == 1) {									// decode to wav
			sFileOut = sDirOut + fiL.baseName() + ".wav";
			slArgs.append(sFileOut);							// add argument: destination file, full path n name
			if (bWavDoNotReC == true) {							// do not re-create waves that already exist
				QFileInfo fiL2(sFileOut);
				if (fiL2.exists() == true) {
					bSkip = true;
				} else {
					bSkip = false;
				}
			}
		} else if (iLameCmd == 2) {							// encode to mp3
			sD = fiL.baseName();								// modify destination file name
			if (bWavDoNotReC == true) {								// randomize file names by prepending 00-
				sD = QString::number(qrand() % ((99 + 1) - 0) + 0) + "-" + sD;
			}
			if (bDelSpaces == true) {								// delete spaces in output file name
				sD.remove(QChar(' '), Qt::CaseInsensitive);
			}
			if (bShorten == true) {									// shorten file names to 8 characters long
				sD = sD.left(8);
			}
			sFileOut = sDirOut + sD + ".mp3";
			slArgs.append(sFileOut);							// add argument: destination file, full path n name
		} else {												// this can never happen, already handled above
		}
		
		if (bSkip == true) {
			fpLD.src = sFileSrc;
			fpLD.out = sFileOut;
			liLFilePairs.append(fpLD);			// add existing mp3 n wav name pair, if we do not re-create as wav already exists
			procLoop();
		} else {
			if (fiL.fileName().length() > 36) {					// limit GUI output to 30 character length
				emit sigLameFileProc(fiL.fileName().left(18) + "..." + fiL.fileName().right(18));		// limit GUI update to 30 character length
			} else {
				emit sigLameFileProc(fiL.fileName());
			}
			procRun();
		}

	} else {
		procLoopEnded();
	}
}

void lame::procRun()						// start qprocess
{
//cout << "lame debug procRun: processing : " << slArgs.at(1).toStdString() << endl;										// *****  debug  *****
	qpLame = new QProcess(this);
	connect(qpLame, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(procEnd(int, QProcess::ExitStatus)));
    connect(qpLame, SIGNAL(readyReadStandardOutput()), this, SLOT(procStdOut()));
    connect(qpLame, SIGNAL(readyReadStandardError()), this, SLOT(procStdErr()));
	qpLame->start("lame", slArgs);
}

void lame::procStdOut()						// read qprocess stdout
{
	aOutput = qpLame->readAllStandardOutput();       // read byte array
	if (!aOutput.isEmpty()) {
		sIn = QString( aOutput ).trimmed();
		cout << "procStdOut: sIn: " << sIn.toStdString() << endl;													// *****  debug  *****
	}
}

void lame::procStdErr()						// read qprocess stderr
{
	aOutput = qpLame->readAllStandardOutput();       // read byte array
	if (!aOutput.isEmpty()) {
		sIn = QString( aOutput ).trimmed();
		cout << "procStdErr: sIn: " << sIn.toStdString() << endl;													// *****  debug  *****
	}
}

void lame::procEnd(int iD, QProcess::ExitStatus iD2)	// qprocess ended, start next if still program to check for
{
//cout << "lame procEnd: sFileOut = " << sFileOut.toStdString() << endl;											// *****  debug  *****	
	QFileInfo fiL3(sFileOut);
	if (fiL3.exists() != true) {
//cout << "lame procEnd: check for newly created file failed : " << fiL3.absoluteFilePath().toStdString() << endl;											// *****  debug  *****
//cout << "lame procEnd: iLameCmd = " << QString::number(iLameCmd).toStdString()  << endl;											// *****  debug  *****
		slReturn.append(slFilesToProc.at(iLoop));			// this file could not be processed successfully (decoding or encoding)
	} else {
		fpLD.src = sFileSrc;
		fpLD.out = sFileOut;
		liLFilePairs.append(fpLD);
	}

	procLoop();
}

void lame::procKill()						// kill qprocess, asked to stop
{
	qpLame->kill();								// need to check whether qpLame exists and running?
}

void lame::procLoopEnded()					// all files processed, return
{
//cout << "lame procLoopEnded: reached ! " << endl;											// *****  debug  *****
//cout << "lame procLoopEnded: slReturn count = " << QString::number(slReturn.count()).toStdString() << endl;											// *****  debug  *****
	if (slReturn.count() > 0) {
		cout << "lame procLoopEnded: slReturn at 0 = " << slReturn.at(0).toStdString() << endl;											// *****  debug  *****
	}

	if (iLameCmd == 1) {
//cout << "lame procLoopEnded: liLFilePairs.count() = " << QString::number(liLFilePairs.count()).toStdString() << endl;											// *****  debug  *****
		emit sigLameDecodeEnded(slReturn, liLFilePairs);
	} else if (iLameCmd == 2) {
		emit sigLameEncodeEnded(slReturn, liLFilePairs);
	}
}


#include "lame.moc"
